<?php

namespace Litepie\Media\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litepie\Media\Http\Requests\MediaRequest;
use Litepie\Media\Interfaces\MediaRepositoryInterface;
use Litepie\Media\Models\Media;

/**
 * Resource controller class for media.
 */
class MediaResourceController extends BaseController
{

    /**
     * Initialize media resource controller.
     *
     * @param type MediaRepositoryInterface $media
     *
     * @return null
     */
    public function __construct(MediaRepositoryInterface $media)
    {
        parent::__construct();
        $this->repository = $media;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litepie\Media\Repositories\Criteria\MediaResourceCriteria::class);
    }

    /**
     * Display a list of media.
     *
     * @return Response
     */
    public function index(MediaRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litepie\Media\Repositories\Presenter\MediaPresenter::class)
                ->$function();
        }

        $media = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('media::media.names'))
            ->view('media::media.index', true)
            ->data(compact('media', 'view'))
            ->output();
    }

    /**
     * Display media.
     *
     * @param Request $request
     * @param Model   $media
     *
     * @return Response
     */
    public function show(MediaRequest $request, Media $media)
    {

        if ($media->exists) {
            $view = 'media::media.show';
        } else {
            $view = 'media::media.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('media::media.name'))
            ->data(compact('media'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new media.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(MediaRequest $request)
    {

        $media = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('media::media.name')) 
            ->view('media::media.create', true) 
            ->data(compact('media'))
            ->output();
    }

    /**
     * Create new media.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MediaRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $media                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('media::media.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('media/media/' . $media->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/media/media'))
                ->redirect();
        }

    }

    /**
     * Show media for editing.
     *
     * @param Request $request
     * @param Model   $media
     *
     * @return Response
     */
    public function edit(MediaRequest $request, Media $media)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('media::media.name'))
            ->view('media::media.edit', true)
            ->data(compact('media'))
            ->output();
    }

    /**
     * Update the media.
     *
     * @param Request $request
     * @param Model   $media
     *
     * @return Response
     */
    public function update(MediaRequest $request, Media $media)
    {
        try {
            $attributes = $request->all();

            $media->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('media::media.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('media/media/' . $media->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('media/media/' . $media->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the media.
     *
     * @param Model   $media
     *
     * @return Response
     */
    public function destroy(MediaRequest $request, Media $media)
    {
        try {

            $media->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('media::media.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('media/media/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('media/media/' . $media->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple media.
     *
     * @param Model   $media
     *
     * @return Response
     */
    public function delete(MediaRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('media::media.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('media/media'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/media/media'))
                ->redirect();
        }

    }

    /**
     * Restore deleted media.
     *
     * @param Model   $media
     *
     * @return Response
     */
    public function restore(MediaRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('media::media.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/media/media'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/media/media/'))
                ->redirect();
        }

    }

}
